import fs from 'fs'
import sharp from 'sharp'
import ffmpeg from 'fluent-ffmpeg'
import { resizeOptions, thumbnailOptions } from './constants'

export function saveStreamToFile(stream: any, path: [string, string]) {
  const filePath = path.join('/')
  return new Promise((resolve, reject) => {
    const file = fs.createWriteStream(filePath)

    file.on('finish', resolve)
    file.on('error', (error) => {
      fs.unlink(filePath, () => {
        reject(error)
      })
    })
    stream.on('error', (error: any) => {
      file.destroy(error)
      reject(error)
    })
    stream.pipe(file)
  })
}

export async function imageResizeConvertJpg(inputPath: string, outputPath: string) {
  const { maxWidth, maxHeight, options } = resizeOptions
  await sharp(inputPath)
    .resize(maxWidth, maxHeight, options)
    .jpeg()
    .toFile(outputPath)
  const metadata = await sharp(outputPath).metadata()
  return { width: metadata.width, height: metadata.height }
}

export function createThumbnail(image: string, thumbnail: string) {
  const { maxWidth, maxHeight, options } = thumbnailOptions
  return sharp(image)
    .resize(maxWidth, maxHeight, options)
    .jpeg()
    .toFile(thumbnail)
}

export function videoScreenshot(video: string, path: string) {
  return new Promise((resolve, reject) => {
    // ffmpeg(video)
    //   .screenshot({ folder, filename: filename, ...screenshotOptions })
    //   .on('end', () => {
    //     fs.promises.rename(`${thumbnail}.png`, thumbnail)
    //     resolve(thumbnail)
    //   })
    //   .on('error', (error) => reject(error))
    ffmpeg(video)
      .inputOption('-itsscale', '0.15')
      .outputOption(
        '-vf',
        'fps=5,scale=200:-1:flags=lanczos,split[s0][s1];[s0]palettegen[p];[s1][p]paletteuse',
        '-loop',
        '-1'
      )
      .save(path)
      .on('end', () => resolve(path))
      .on('error', (error) => reject(error))
  })
}

export function mediumMetadata(medium: string): {} {
  return new Promise((resolve, reject) => {
    ffmpeg.ffprobe(medium, (error, metadata) => {
      if (error) reject(error)
      else {
        const videoStreams = metadata.streams.filter((stream) => stream.codec_type === 'video')
        const audioStreams = metadata.streams.filter((stream) => stream.codec_type === 'audio')
        if (audioStreams.length > 0) {
          const audioStream = audioStreams[0]
          resolve({
            duration: parseFloat(audioStream.duration || '0'),
          })
        } else if (videoStreams.length > 0) {
          const videoStream = videoStreams[0]
          resolve({
            duration: parseFloat(videoStream.duration || '0'),
            width: videoStream.width,
            height: videoStream.height,
          })
        } else {
          resolve({})
        }
      }
    })
  })
}

export async function processImage(stream: any, path: [string, string]) {
  const filePath = path.join('/')
  const thumbnail = `${filePath}.jpg`
  const metadata = await imageResizeConvertJpg(stream, filePath)
  await createThumbnail(filePath, thumbnail)
  return { thumbnail, ...metadata }
}

export async function processVideo(stream: any, path: [string, string]) {
  const filePath = path.join('/')
  const thumbnail = `${filePath}.gif`
  await saveStreamToFile(stream, path)
  await videoScreenshot(filePath, thumbnail)
  const metadata = await mediumMetadata(filePath)
  return { thumbnail, ...metadata }
}

export async function processAudio(stream: any, path: [string, string]) {
  const filePath = path.join('/')
  await saveStreamToFile(stream, path)
  const metadata = await mediumMetadata(filePath)
  return { ...metadata }
}
