import ExerciseInputs from './ExerciseInputs'
import { TextInput } from '../../form'
import { TExerciseInstance, TExercise } from '../types'
import theme from '../../styles/theme'
import { useState, ChangeEvent } from 'react'
import { emptyExercise } from '../utils'
import { FontAwesomeIcon } from '@fortawesome/react-fontawesome'
import { faChevronRight, faChevronDown } from '@fortawesome/free-solid-svg-icons'
import ExerciseSelector from './ExerciseSelector'

const ExerciseInstanceInputs = ({
  value,
  name,
  onChange,
  className = 'ei-fields',
}: {
  value: TExerciseInstance
  name: string
  onChange: GenericEventHandler
  className?: string
}) => {
  const [show, setShow] = useState(true)
  const [source, setSource] = useState<'new' | 'existing'>('new')
  const [newItem, setNewItem] = useState(emptyExercise())
  const [existingItem, setExistingItem] = useState<undefined | TExercise>()

  function handleSourceChange(event: ChangeEvent<HTMLInputElement>) {
    const { target } = event
    if (target.value === 'new') {
      if (!value.exercise?.id.startsWith('++')) setExistingItem(value.exercise)
      setSource('new')
      onChange({ target: { type: 'custom', name: `${name}.exercise`, value: newItem } })
    } else if (target.value === 'existing') {
      if (value.exercise?.id.startsWith('++')) setNewItem(value.exercise)
      setSource('existing')
      onChange({ target: { type: 'custom', name: `${name}.exercise`, value: existingItem } })
    }
  }

  return (
    <fieldset className={className}>
      {!value.id.startsWith('++') && (
        <div className='ei-info'>
          <div>Exercise Instance ID{value.id}</div>
          <div>Created at:</div>
        </div>
      )}
      <TextInput
        name={`${name}.order`}
        label='Order'
        value={value.order}
        type='number'
        onChange={onChange}
        className='ei-order'
      />
      <TextInput
        name={`${name}.rounds`}
        label='Repetitions'
        value={value.repetitions}
        type='number'
        onChange={onChange}
        className='ei-rounds'
      />
      <TextInput
        name={`${name}.variation`}
        label='Variation'
        value={value.variation}
        onChange={onChange}
        className='ei-variation'
      />
      <div className='ei-exercise'>
        <FontAwesomeIcon
          icon={show ? faChevronDown : faChevronRight}
          height={16}
          onClick={() => setShow(!show)}
        />
        <input
          type='radio'
          name='source'
          id='source-new'
          value='new'
          checked={source === 'new'}
          onChange={handleSourceChange}
        />
        <label htmlFor='source-new'>New exercise</label>
        <input
          type='radio'
          name='source'
          id='source-existing'
          value='existing'
          checked={source === 'existing'}
          onChange={handleSourceChange}
        />
        <label htmlFor='source-existing'>Existing exercise</label>
        {value.exercise ? (
          <div>Exercise: {value.exercise.name ?? value.exercise.description?.substr(0, 50)}</div>
        ) : (
          <div>No exercise selected</div>
        )}

        {show &&
          (source === 'new' ? (
            <ExerciseInputs name={`${name}.exercise`} value={value.exercise} onChange={onChange} />
          ) : source === 'existing' ? (
            <ExerciseSelector
              name={`${name}.exercise`}
              value={value.exercise}
              label='Existing exercise'
              onChange={onChange}
            />
          ) : (
            <p>Please select a exercise</p>
          ))}
      </div>

      <style jsx>{`
        .ei-exercise > label,
        .ei-exercise > input[type='radio'] {
          display: inline;
          width: auto;
        }
        @media (min-width: ${theme.midsize}) {
          .${className} {
            display: grid;
            grid-template-areas:
              'info info info info'
              'order  rounds variation variation'
              'exercise  exercise  exercise exercise'
              'button button button button';
            grid-template-columns: repeat(4, 1fr);
          }

          .${className} :global(.ei-order) {
            grid-area: order;
          }
          .${className} :global(.ei-rounds) {
            grid-area: rounds;
          }
          .${className} :global(.ei-variation) {
            grid-area: variation;
          }
          .${className} :global(.ei-exercise) {
            grid-area: exercise;
          }
          .${className} :global(.ei-button) {
            grid-area: button;
          }
        }
      `}</style>
    </fieldset>
  )
}

export default ExerciseInstanceInputs
